/*  nanowasp, an emulator for the microbee 128k
 *  Copyright (C) 2000,2001  David G. Churchill <froods@alphalink.com.au>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 *  Keyboard Module
 *
 *
 *  Emulates the keyboard by triggering the lpen input
 *  on the CRTC when mem_addr matches a sad (depressed) key.
 *
 *
 *  To do:
 *    - make keymap user reconfigurable
 */


#include <stdio.h>
#include <stdlib.h>
#include <allegro.h>

#include "keyb.h"
#include "crtc.h"
#include "vdu.h"
#include "nanowasp.h"


static struct
{
   int key, type;
}  keymap[] =
  {  { KEY_QUOTE,       KB_NORMAL },
     { KEY_A,           KB_NORMAL },
     { KEY_B,           KB_NORMAL },
     { KEY_C,           KB_NORMAL },
     { KEY_D,           KB_NORMAL },
     { KEY_E,           KB_NORMAL },
     { KEY_F,           KB_NORMAL },
     { KEY_G,           KB_NORMAL },
     { KEY_H,           KB_NORMAL },
     { KEY_I,           KB_NORMAL },
     { KEY_J,           KB_NORMAL },
     { KEY_K,           KB_NORMAL },
     { KEY_L,           KB_NORMAL },
     { KEY_M,           KB_NORMAL },
     { KEY_N,           KB_NORMAL },
     { KEY_O,           KB_NORMAL },
     { KEY_P,           KB_NORMAL },
     { KEY_Q,           KB_NORMAL },
     { KEY_R,           KB_NORMAL },
     { KEY_S,           KB_NORMAL },
     { KEY_T,           KB_NORMAL },
     { KEY_U,           KB_NORMAL },
     { KEY_V,           KB_NORMAL },
     { KEY_W,           KB_NORMAL },
     { KEY_X,           KB_NORMAL },
     { KEY_Y,           KB_NORMAL },
     { KEY_Z,           KB_NORMAL },
     { KEY_OPENBRACE,   KB_NORMAL },
     { KEY_BACKSLASH,   KB_NORMAL },
     { KEY_CLOSEBRACE,  KB_NORMAL },
     { KEY_TILDE,       KB_NORMAL },
     { KEY_DEL,         KB_NORMAL | KB_EXTENDED },
     { KEY_0,           KB_NORMAL },
     { KEY_1,           KB_NORMAL },
     { KEY_2,           KB_NORMAL },
     { KEY_3,           KB_NORMAL },
     { KEY_4,           KB_NORMAL },
     { KEY_5,           KB_NORMAL },
     { KEY_6,           KB_NORMAL },
     { KEY_7,           KB_NORMAL },
     { KEY_8,           KB_NORMAL },
     { KEY_9,           KB_NORMAL },
     { KEY_COLON,       KB_NORMAL },
     { KEY_EQUALS,      KB_NORMAL },
     { KEY_COMMA,       KB_NORMAL },
     { KEY_MINUS,       KB_NORMAL },
     { KEY_STOP,        KB_NORMAL },
     { KEY_SLASH,       KB_NORMAL },
     { KEY_ESC,         KB_NORMAL },
     { KEY_BACKSPACE,   KB_NORMAL },
     { KEY_TAB,         KB_NORMAL },
     { KEY_ENTER,       KB_EXTENDED },  /* LF */  /* FIXME readkey prob matches both enters */
     { KEY_ENTER,       KB_NORMAL },
     { KEY_CAPSLOCK,    KB_NORMAL },
     { KEY_PLUS_PAD,    KB_NORMAL | KB_EXTENDED }, /* break */
     { KEY_SPACE,       KB_NORMAL },
     { KEY_F1,          KB_NORMAL }, /* 61 */
     { KEY_CONTROL,     KB_NORMAL },
     { KEY_F2,          KB_NORMAL }, /* 62 */
     { KEY_F5,          KB_NORMAL }, /* 65 */
     { KEY_F4,          KB_NORMAL }, /* 64 */
     { KEY_F3,          KB_NORMAL }, /* 63 */
     { KEY_F6,          KB_NORMAL }, /* 66 */
     { KEY_LSHIFT,      KB_NORMAL } };



int keyb_init(void)
{
   install_keyboard();                 /* PORTME allegro */

   return 0;
}


int keyb_deinit(void)
{
   remove_keyboard();                  /* PORTME allegro */

   return 0;
}


int keyb_reset(void)
{
/*   clear_keybuf();  this makes keys not work over reset */

   return 0;
}



void keyb_handler(int addr)
{
   int scan;


   scan = (addr >> 4) & 0x3F;

   if (debuglvl > 0)
      fprintf(debuglog, "keyb: addr = 0x%04x, scan = 0x%02x\n",
              addr, scan);

   if (key[keymap[scan].key] & keymap[scan].type)
      crtc_lpen(addr);
}



void keyb_checkall(void)
{
   int i;


   if (!latchrom)
      for (i=0x3F; i>=0; i--)
         if (key[keymap[i].key] & keymap[i].type)
         {
            crtc_lpen(i << 4);
            break;
         }
}

